#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <math.h>
#include "main.h"
#include "Graphics.h"
#include "System.h"

#define SCREEN_WIDTH    640
#define SCREEN_HEIGHT   480
#define NB_POINTS 20

struct SPoint
{
    int     x, y;
    Color   col;
};

int main(int argc, char* argv[])
{
    SPoint  points[NB_POINTS];
    float radius = 1.0;

    // init the window
    gfx.init("Voronoi 2", SCREEN_WIDTH, SCREEN_HEIGHT);
    gfx.init2D();
    gfx.clearScreen(Color(0, 0, 0, SDL_ALPHA_OPAQUE));

    // init the points
    srand(time(NULL));

    for (int i = 0; i < NB_POINTS; ++i)
    {
        points[i].x = rand() % SCREEN_WIDTH;
        points[i].y = rand() % SCREEN_HEIGHT;
        points[i].col = Color((rand() % 128) + 128,
                              (rand() % 128) + 128,
                              (rand() % 128) + 128);
    }

    while (sys.isQuitRequested() == false)
    {
        sys.processEvents();

        // draw circles
        for (int y = 0; y < SCREEN_HEIGHT; ++y)
            for (int x = 0; x < SCREEN_WIDTH; ++x)
            {
                int     nearest = 0;
                Sint32  nearestDist = 1000000000;

                for (int i = 0; i < NB_POINTS; ++i)
                {
                    Sint32  dx = points[i].x - x;
                    Sint32  dy = points[i].y - y;
                    Sint32  dist = dx * dx + dy * dy;

                    if (dist < nearestDist)
                    {
                        nearest = i;
                        nearestDist = dist;
                    }
                }

                if (sqrt(nearestDist) <= radius)
                    gfx.setPixel(x, y, points[nearest].col);
            }

        // draw points
        for (int i = 0; i < NB_POINTS; ++i)
        {
            gfx.setPixel(points[i].x    , points[i].y    , Color(1, 1, 1, SDL_ALPHA_OPAQUE));
            gfx.setPixel(points[i].x    , points[i].y - 1, Color(1, 1, 1, SDL_ALPHA_OPAQUE));
            gfx.setPixel(points[i].x    , points[i].y + 1, Color(1, 1, 1, SDL_ALPHA_OPAQUE));
            gfx.setPixel(points[i].x - 1, points[i].y    , Color(1, 1, 1, SDL_ALPHA_OPAQUE));
            gfx.setPixel(points[i].x + 1, points[i].y    , Color(1, 1, 1, SDL_ALPHA_OPAQUE));
        }

        // increase radius
        radius += 1.0;

        gfx.render();
        sys.wait(50);
    }

    gfx.quit();

    return EXIT_SUCCESS;
}
